# Path to the log file
$logFilePath = "$PSScriptRoot\ScheduleRebootLog.txt"

function Write-Log {
    param (
        [string]$message
    )
    $timestamp = Get-Date -Format "yyyy-MM-dd HH:mm:ss"
    
    # Retry mechanism to avoid conflicts in log writing
    $maxRetries = 5
    $retryCount = 0
    $success = $false

    while (-not $success -and $retryCount -lt $maxRetries) {
        try {
            Add-Content -Path $logFilePath -Value "$timestamp - $message"
            $success = $true
        }
        catch {
            Write-Host "Log file is currently in use, retrying in 1 second..."
            Start-Sleep -Seconds 1
            $retryCount++
        }
    }

    if (-not $success) {
        Write-Host "Failed to write to log file after $maxRetries retries."
    }
}

# Start logging
Write-Log "--- Starting Task Scheduling Process ---"

try {
    # Check and log current execution policy
    $currentExecutionPolicy = Get-ExecutionPolicy
    Write-Log "Current execution policy: $currentExecutionPolicy"

    # Adjust execution policy if needed
    if ($currentExecutionPolicy -ne "RemoteSigned") {
        Set-ExecutionPolicy RemoteSigned -Scope Process -Force
        Write-Log "Execution policy set to RemoteSigned temporarily."
    }

    # Check if the task already exists and delete it
    Write-Log "Checking if task already exists..."
    $taskExists = schtasks /Query /TN "Scheduled Reboot Event - Black Screen fix - 3C Technology" 2>&1 | Select-String "INFO: Task does not exist" -NotMatch
    if ($taskExists) {
        Write-Log "Task exists. Attempting to delete the existing task..."
        schtasks /Delete /TN "Scheduled Reboot Event - Black Screen fix - 3C Technology" /F
        Write-Log "Existing task deleted."
    }

    # Define the action to restart the computer
    Write-Log "Defining shutdown action for task..."
    $action = New-ScheduledTaskAction -Execute 'shutdown' -Argument '/r /f /t 0'
    Write-Log "Shutdown action defined successfully."

    # Define the trigger for daily at 4:00 AM
    Write-Log "Defining trigger for daily reboot at 4:00 AM..."
    $trigger = New-ScheduledTaskTrigger -Daily -At 4:00AM
    Write-Log "Trigger defined successfully."

    # Set the task to run under the current user
    $currentUser = (whoami)
    Write-Log "Running task under user: $currentUser"
    $principal = New-ScheduledTaskPrincipal -UserId $currentUser -LogonType Interactive
    Write-Log "Task principal defined successfully."

    # Set task settings (e.g., time limit)
    Write-Log "Setting additional task parameters..."
    $settings = New-ScheduledTaskSettingsSet
    $settings.StartWhenAvailable = $true
    $settings.ExecutionTimeLimit = "PT30M"
    Write-Log "Task settings defined successfully."

    # Create the scheduled task
    Write-Log "Attempting to register scheduled task..."
    Register-ScheduledTask -TaskName "Scheduled Reboot Event - Black Screen fix - 3C Technology" -Action $action -Trigger $trigger -Principal $principal -Settings $settings -ErrorAction Stop
    Write-Log "Scheduled task created successfully."

    # Verify task creation using schtasks
    $schtaskOutput = schtasks /Query /TN "Scheduled Reboot Event - Black Screen fix - 3C Technology" 2>&1
    if ($schtaskOutput -match "INFO: Task does not exist") {
        Write-Log "ERROR: Task does not exist."
    } else {
        Write-Log "Task verification: Task exists."
    }

}
catch {
    Write-Log "An exception occurred: $_"
}

Write-Log "--- Task Scheduling Process Ended ---"
Write-Host "Press Enter to close this window..."
Read-Host
